﻿/*
 * Copyright (c) eHealth
 */
using System;
using be.cin.nip.async.generic;
using be.ehealth.business.mycarenetdomaincommons.domain;
using be.ehealth.businessconnector.genericasync.domain;
using be.ehealth.technicalconnector.utils;
using be.fgov.ehealth.technicalconnector.signature.domain;
using be.fgov.ehealth.technicalconnector.tests.utils;
using be.vlaanderen.mercurius.insurability.schemas.v1;
using be.vlaanderen.mercurius.mohm.schemas.v1;
using java.util;
using java.util.concurrent;
using NUnit.Framework;
using org.slf4j;

namespace be.ehealth.businessconnector.vsbnet.async.session
{
	public class VsbNetGenericAsyncMethod
	{
		private static Logger LOG = LoggerFactory.getLogger(typeof(VsbNetGenericAsyncMethod));
		
		private static int SECONDS_WAITING = 60;

		/******************
		 *
		 * POST Methods
		 *
		 *******************/

		public static void do_POST_CreateApplication(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.createApplication((EMOHMDF101CreateApplicationRequest)ConnectorXmlUtils
				                   .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
				                             typeof(EMOHMDF101CreateApplicationRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCreateApplication(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCreateApplication(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_ConsultApplication(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.consultApplication((EMOHMDF103ConsultapplicationRequest)ConnectorXmlUtils
				                    .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
				                              typeof(EMOHMDF103ConsultapplicationRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultApplication(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultApplication(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_ReportDelivery(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.reportDelivery((EMOHMDF105ReportDeliveryRequest)ConnectorXmlUtils
				                .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
				                          typeof(EMOHMDF105ReportDeliveryRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getReportDelivery(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getReportDelivery(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_ConsultSupportAndRepair(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.consultSupportAndRepair((EMOHMDF104ConsultSupportAndRepairRequest)ConnectorXmlUtils
				                         .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
				                                   typeof(EMOHMDF104ConsultSupportAndRepairRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultSupportAndRepair(GetRequest.newBuilder().withReference(reference).build());
			VsbNetSessionServiceFactory.getVsbNetMohmService().confirmTAck((ProcessedTAckResponse)response.getTAckResponses().get(0));
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultSupportAndRepair(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_ConsultSupportAndRepairList(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().consultSupportAndRepairList((EMOHMDF112ConsultSupportAndRepairListRequest)ConnectorXmlUtils
			                                                                                                                             .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
			                                                                                                                                       typeof(EMOHMDF112ConsultSupportAndRepairListRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultSupportAndRepairList(GetRequest.newBuilder().withReference(reference).build());
			VsbNetSessionServiceFactory.getVsbNetMohmService().confirmTAck((ProcessedTAckResponse)response.getTAckResponses().get(0));
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultSupportAndRepairList(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_CancelApplication(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().cancelApplication((EMOHMDF111CancelApplicationRequest)ConnectorXmlUtils
			                                                                                                                   .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
			                                                                                                                             typeof(EMOHMDF111CancelApplicationRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCancelApplication(GetRequest.newBuilder().withReference(reference).build());
			VsbNetSessionServiceFactory.getVsbNetMohmService().confirmTAck((ProcessedTAckResponse)response.getTAckResponses().get(0));
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCancelApplication(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_CreateFictionalApplication(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.createFictionalApplication((EMOHMDF102CreateFictionalApplicationRequest)ConnectorXmlUtils
				                            .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
				                                      typeof(EMOHMDF102CreateFictionalApplicationRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCreateFictionalApplication(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCreateFictionalApplication(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_RefusedInvoice(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.postInvoiceList(ConnectorIOUtils.getResourceAsByteArray(requestLocation), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceFileRefused(GetRequest.newBuilder().withReference(reference).build());
			validateXadesTOfTAckResponseWithManifest(processedPostResponse, response);
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceFileRefused(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_AcceptedInvoice(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.postInvoiceList(ConnectorIOUtils.getResourceAsByteArray(requestLocation), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceFileAccepted(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceAccepted1(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceAccepted2(GetRequest.newBuilder().withReference(reference).build());
			confirmTacks(response);
			response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceAccepted1(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
			response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceAccepted2(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		public static void do_POST_DetermineInsurability(String requestLocation) {
			ProcessedPostResponse processedPostResponse = VsbNetSessionServiceFactory.getVsbNetInsurabilityService()
				.determineInsurability((WZCMHDF001DetermineInsurabilityRequest)ConnectorXmlUtils
				                       .toObject(ConnectorIOUtils.getResourceAsByteArray(requestLocation),
				                                 typeof(WZCMHDF001DetermineInsurabilityRequest)), generateInputReference());
			String reference = processedPostResponse.getPostResponse().getReturn().getReference();
			waitForTack();
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInsurabilityService()
				.getDetermineInsurability(GetRequest.newBuilder().withReference(reference).build());
			confirmTack((ProcessedTAckResponse)response.getTAckResponses().get(0));
			response = VsbNetSessionServiceFactory.getVsbNetInsurabilityService()
				.getDetermineInsurability(GetRequest.newBuilder().withReference(reference).build());
			assertTackRemoved(response.getTAckResponses());
		}

		/******************
		 *
		 * GET Methods
		 *
		 *******************/

		public static void do_GET_CreateApplication(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCreateApplication(GetRequest.newBuilder().withDefaults().build());

			assertCreateApplication(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ReportDelivery(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getReportDelivery(GetRequest.newBuilder().withDefaults().build());

			assertReportDelivery(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ConsultSupportAndRepair(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultSupportAndRepair(GetRequest.newBuilder().withDefaults().build());

			assertConsultSupportAndRepair(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_CancelApplication(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCancelApplication(GetRequest.newBuilder().withDefaults().build());

			assertCancelApplication(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ConsultSupportAndRepairList(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultSupportAndRepairList(GetRequest.newBuilder().withDefaults().build());

			assertConsultSupportAndRepairList(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ConsultApplication(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getConsultApplication(GetRequest.newBuilder().withDefaults().build());

			assertConsultApplication(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_CreateFictionalApplication(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getCreateFictionalApplication(GetRequest.newBuilder().withDefaults().build());

			assertCreateFictionalApplication(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_PushApplicationView(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getPushApplicationView(GetRequest.newBuilder().withDefaults().build());

			assertPushApplicationView(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_PushDecision(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetMohmService()
				.getPushDecision(GetRequest.newBuilder().withDefaults().build());

			assertPushDecision(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ListInvoiceFileAccepted(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceFileAccepted(GetRequest.newBuilder().withDefaults().build());

			assertInvoiceList(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ListInvoiceAccepted1(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceAccepted1(GetRequest.newBuilder().withDefaults().build());

			assertInvoiceList(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ListInvoiceAccepted2(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceAccepted2(GetRequest.newBuilder().withDefaults().build());

			assertInvoiceList(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ListInvoiceRefused(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceRefused(GetRequest.newBuilder().withDefaults().build());

			assertInvoiceList(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_ListInvoiceFileRefused(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInvoiceService()
				.getListInvoiceFileRefused(GetRequest.newBuilder().withDefaults().build());

			assertInvoiceList(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		public static void do_GET_DetermineInsurability(String responseLocation, Boolean toConfirm) {
			ProcessedGetResponse response = VsbNetSessionServiceFactory.getVsbNetInsurabilityService()
				.getDetermineInsurability(GetRequest.newBuilder().withDefaults().build());

			assertDetermineInsurability(responseLocation, response);

			if (toConfirm) {
				ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(response);
				Assert.NotNull(confirmResponse);
			}
		}

		/******************
		 *
		 * Other Methods
		 *
		 *******************/

		private static void assertCreateApplication(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF101CreateApplicationResponse expected = (EMOHMDF101CreateApplicationResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF101CreateApplicationResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF101CreateApplicationResponse businessResponse = (EMOHMDF101CreateApplicationResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getCreateApplication().getVSBReference().Equals(businessResponse.getCreateApplication().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertConsultApplication(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF103ConsultApplicationResponse expected = (EMOHMDF103ConsultApplicationResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF103ConsultApplicationResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF103ConsultApplicationResponse businessResponse = (EMOHMDF103ConsultApplicationResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getConsultApplication().getVSBReference().Equals(businessResponse.getConsultApplication().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertReportDelivery(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF105ReportDeliveryResponse expected = (EMOHMDF105ReportDeliveryResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF105ReportDeliveryResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF105ReportDeliveryResponse businessResponse = (EMOHMDF105ReportDeliveryResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getReportDelivery().getVSBReference().Equals(businessResponse.getReportDelivery().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertConsultSupportAndRepair(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF104ConsultSupportAndRepairResponse expected = (EMOHMDF104ConsultSupportAndRepairResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF104ConsultSupportAndRepairResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF104ConsultSupportAndRepairResponse businessResponse = (EMOHMDF104ConsultSupportAndRepairResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getConsultSupportAndRepair().getVSBReference().Equals(businessResponse.getConsultSupportAndRepair().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}
		private static void assertCancelApplication(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF111CancelApplicationResponse expected = (EMOHMDF111CancelApplicationResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF111CancelApplicationResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF111CancelApplicationResponse businessResponse = (EMOHMDF111CancelApplicationResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getCancelApplication().getVSBReference().Equals(businessResponse.getCancelApplication().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertConsultSupportAndRepairList(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF112ConsultSupportAndRepairListResponse expected = (EMOHMDF112ConsultSupportAndRepairListResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF112ConsultSupportAndRepairListResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF112ConsultSupportAndRepairListResponse businessResponse = (EMOHMDF112ConsultSupportAndRepairListResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getMessageMetadata().getSSIN().Equals(businessResponse.getMessageMetadata().getSSIN())
				    && expected.getMessageMetadata().getEnterpriseNumber().Equals(businessResponse.getMessageMetadata().getEnterpriseNumber())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertCreateFictionalApplication(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF102CreateFictionalApplicationResponse expected = (EMOHMDF102CreateFictionalApplicationResponse)ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF102CreateFictionalApplicationResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF102CreateFictionalApplicationResponse businessResponse = (EMOHMDF102CreateFictionalApplicationResponse)processedMsgResponse.getBusinessResponse();
				if (expected.getCreateFictionalApplication().getVSBReference().Equals(businessResponse.getCreateFictionalApplication().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertPushDecision(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF106PushDecisionRequest expected = (EMOHMDF106PushDecisionRequest)ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF106PushDecisionRequest));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF106PushDecisionRequest businessResponse = (EMOHMDF106PushDecisionRequest)processedMsgResponse.getBusinessResponse();
				if (expected.getPushDecision().getVSBReference().Equals(businessResponse.getPushDecision().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertPushApplicationView(String expectedResponse, ProcessedGetResponse response) {
			EMOHMDF107PushApplicationViewRequest expected = (EMOHMDF107PushApplicationViewRequest)ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(EMOHMDF107PushApplicationViewRequest));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse)response.getMsgResponses().get(i);
				
				EMOHMDF107PushApplicationViewRequest businessResponse = (EMOHMDF107PushApplicationViewRequest)processedMsgResponse.getBusinessResponse();
				if (expected.getPushApplication().getVSBReference().Equals(businessResponse.getPushApplication().getVSBReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmMessage(processedMsgResponse);
					Assert.NotNull(confirmResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void assertInvoiceList(String expectedResponse, ProcessedGetResponse response) {
			byte[] expected = ConnectorIOUtils.getResourceAsByteArray(expectedResponse);
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse) response.getMsgResponses().get(i);
				
				Assert.AreEqual(BitConverter.ToString(expected), BitConverter.ToString((byte[])processedMsgResponse.getBusinessResponse()));
				Assert.AreEqual(0, processedMsgResponse.getSignatureVerificationResult().getErrors().size(),"SignatureVerificationResult should contain no error");
				Assert.NotNull(processedMsgResponse.getMsgResponse().getXadesT(), "Missing xades");
				Assert.IsTrue(Arrays.equals((byte[]) processedMsgResponse.getBusinessResponse(), processedMsgResponse.getSignedData()),"Signed data and businessResponse content do not match");
			}
		}

		private static void assertDetermineInsurability(String expectedResponse, ProcessedGetResponse response) {
			WZCMHDF001DetermineInsurabilityResponse expected = (WZCMHDF001DetermineInsurabilityResponse) ConnectorXmlUtils.toObject(ConnectorIOUtils.getResourceAsString(expectedResponse), typeof(WZCMHDF001DetermineInsurabilityResponse));
			for (int i = 0; i < response.getMsgResponses().size(); i++) {
				ProcessedMsgResponse processedMsgResponse = (ProcessedMsgResponse) response.getMsgResponses().get(i);
				
				WZCMHDF001DetermineInsurabilityResponse businessResponse = (WZCMHDF001DetermineInsurabilityResponse) processedMsgResponse.getBusinessResponse();
				if (expected.getMessageMetadata().getCareProviderReference().Equals(businessResponse.getMessageMetadata().getCareProviderReference())) {
					XmlAsserter.assertSimilar(expected, businessResponse);
					return;
				}
			}
			failNoMessageResponse();
		}

		private static void failNoMessageResponse() {
			Assert.Fail("No message reponse");
		}

		private static void confirmMsgResponses(ProcessedGetResponse processedGetResponse) {
			ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllMessages(processedGetResponse);
			Assert.NotNull(confirmResponse);
		}

		private static void assertTackRemoved(java.util.List tAckResponses) {
			Assert.AreEqual(0, tAckResponses.size(),"Tack should be removed");
		}

		private static void confirmTacks(ProcessedGetResponse processedGetResponse) {
			ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmAllTAcks(processedGetResponse);
			Assert.NotNull(confirmResponse);
		}

		private static void confirmTack(ProcessedTAckResponse processedTAckResponse) {
			ConfirmResponse confirmResponse = VsbNetSessionServiceFactory.getVsbNetMohmService().confirmTAck(processedTAckResponse);
			Assert.NotNull(confirmResponse);
		}

		private static void waitForTack() {
			TimeUnit.SECONDS.sleep(SECONDS_WAITING);
		}

		private static InputReference generateInputReference() {
			return new InputReference();
		}

		private static void validateXadesTOfTAckResponseWithManifest(ProcessedPostResponse processedPostResponse, ProcessedGetResponse response) {
			ProcessedTAckResponse tackResponse = (ProcessedTAckResponse)response.getTAckResponses().get(0);
			SignatureVerificationResult signatureVerificationResult = VsbNetSessionServiceFactory.getVsbNetInvoiceService().validateXadesTWithManifest(tackResponse.getTAckResponse(), processedPostResponse.getPost().getDetail(), tackResponse.getTAckResponse().getXadesT().getValue());
			Assert.IsTrue(signatureVerificationResult.getErrors().isEmpty());
		}
	}
}
